/** 
 * Thread Parallel QuickSort 
 *
 * see Sedgewick or Cormen+
 *
 */

//package edu.unima.ky.parallel;

import java.io.*;

public class ParQuickSort extends RecursionTask {

    private static int[] A; // QuickSort uses same Array
    private int l, r;

    private static int thresh = 1000;
    private static int numw = 4;
    private static RecursionThreads workers = null;

    private ParQuickSort(int l, int r) {
	this.l = l;
        this.r = r;
    }

    public ParQuickSort(int[] A) {
        this.A = A;
	l = 0;
        r = A.length-1;
        numw = Math.min(numw,r/thresh);
        if (workers == null) {
            workers = new RecursionThreads(numw);
        }
    }


    public synchronized void sort() {
        workers.invoke(this); //cormenQuicksort(0, A.length-1);
        workers.terminate(); 
        workers = null;
	return;
    }


    public int[] getArray() {
	return A;
    }


    public void compute() {
	//System.out.println("compute: l = " + l + ", r = " + r);
        cormenQuicksort(l,r);
	return;
    }

    private int partition(int l, int r) {
	// if (l >= r) return l;
	int x, t, i, j;
        x = A[l]; i = l-1; j = r+1;
        while (true) {
	    do { j--; } while ( A[j] > x );
	    do { i++; } while ( A[i] < x );
            if ( i < j ) {
               t = A[i]; A[i] = A[j]; A[j] = t; 
            } else return j;
        }
    }

    private void cormenQuicksort(int l, int r) {
	if (l >= r) return;
	int q = partition(l,r);

        // sequential recursion 
        if ( (r-l) < thresh ) { 
           cormenQuicksort(l,q); 
           cormenQuicksort(q+1,r);
           return;
        } 

        // recursion in parallel
        ParQuickSort t1 = new ParQuickSort(l,q);
        ParQuickSort t2 = new ParQuickSort(q+1,r);
        concurrent(t1,t2);

	/*
        ParQuickSort[] tr = new ParQuickSort[2];
        tr[0] = new ParQuickSort(l,q);
        tr[1] = new ParQuickSort(q+1,r);
        concurrent(tr);
	*/

	return;
    }

}
