
public class MatMult {


/** 
 * Performs a subtraction operation between two arrays sequentially.
 * @param C, A, B Arrays.
 */
  public void seqdiff(double[][] C, double[][] A, double[][] B) {
      for (int i=0; i < C.length; i++) {
          for (int j=0; j < C[0].length; j++) {
              C[i][j] = A[i][j] - B[i][j];
          }
      }
  }


/**
 * Generates a matrix.
 * The method <CODE>random</CODE> returns a 
 * random number within the intervall [0,1].
 * @param n, m integers, dimension of matrix.
 */
  public double[][] matgen(int n, int m) {

/** 
 * Creates a two dimensional array <CODE>A</CODE> with the dimension n, m.
 */
      double A[][] = new double[n][m];
      for (int i=0; i < n; i++) {
          for (int j=0; j < m; j++) {
              A[i][j] = Math.random();
          }
      }
      return A;
  }

/** 
 * Generates the null-matrix.
 * @param n, m integers, dimension of matrix.
 */
  public double[][] matgen0(int n, int m) {
      double A[][] = new double[n][m];
      for (int i=0; i < n; i++) {
          for (int j=0; j < m; j++) {
              A[i][j] = 0.0;
          }
      }
      return A;
  }

/** 
 * Generates the unit-matrix.
 * @param n, m integers, dimension of matrix.
 */
  public double[][] matgen1(int n, int m) {
      double A[][] = new double[n][m];
      for (int i=0; i < n; i++) {
          for (int j=0; j < m; j++) {
              if (i == j) A[i][j] = 1.0; else A[i][j]= 0.0;
          }
      }
      return A;
  }

/** 
 * Prints out the rows of matrix.
 * @param A  Array. 
 */
  public void matprint(double[][] A) {
      for (int i=0; i < A.length; i++) {
          for (int j=0; j < A[0].length; j++) {
              System.out.print(A[i][j] + " ");
          }
          System.out.println();
      }
  }

/** 
 * Checks if the matrix is approximatly zero.
 * Checks if the absolut value of the matrix's elements are less than
 * the production of the smallest positive value and 1000.
 * @param A Two dimensional array.
 */ 
  public boolean matcheck0(double[][] A) {
      double eps = Double.MIN_VALUE*1000.0;
      for (int i=0; i < A.length; i++) {
          for (int j=0; j < A[0].length; j++) {
              if ( Math.abs(A[i][j]) > eps) return false;
          }
      }
      return true;
  }

/**
 * main
 */
  public static void main(String[] args) {

        int n = 2, m = 4;
        boolean prnt = false;

        try { n = Integer.parseInt(args[0]); }
        catch (Exception e) { }
        try { m = Integer.parseInt(args[1]); }
        catch (Exception e) { }
        try { prnt = Boolean.valueOf(args[2]).booleanValue(); }
        catch (Exception e) { }

        MatMult x = new MatMult();

        MMInf seq = new SeqMult();
        MMInf par = new ParMult();

        double[][] A = x.matgen1(n,m);
        double[][] B = x.matgen(m,n);
        double[][] C = x.matgen0(n,n);
        double[][] D = x.matgen0(n,n);
 
        System.out.println("A = ");
        if (prnt) x.matprint(A);
        System.out.println("");

        System.out.println("B = ");
        if (prnt) x.matprint(B);
        System.out.println("");

        System.out.println("C = ");
        long tm = System.currentTimeMillis();
        seq.multiply(C,A,B);
        tm = System.currentTimeMillis() - tm; 
        System.out.println(tm + "ms");
        tm = System.currentTimeMillis();
        par.multiply(C,A,B);
        tm = System.currentTimeMillis() - tm; 
        System.out.println(tm + "ms");
        if (prnt) x.matprint(C);
        System.out.println("");

        System.out.println("D = ");
        x.seqdiff(D,B,C);
        if (prnt) x.matprint(D);
        System.out.println("");
        System.out.println("D is zero = " + x.matcheck0(D) );
    }

}
