//package edu.unima.ky.parallel;

/** 
  * Bounded buffer  
  * @author Akitoshi Yoshida
  * @author Heinz Kredel. 
  */ 
public class BoundedBuffer extends Object {

  /** 
    * The buffer storage of objects.
    */ 
  private Object[] buffer;

  /** 
    * The size of the buffer.
    */
  private int size;

  /** 
    * The position of the first filled cell. 
    */ 
 private int front;

  /** 
    * The position of the first empty cell. 
    */ 
  private int rear;

  /** 
    * A semaphore to indicate an empty buffer. 
    */ 
  private Sema empty;

  /** 
    * A semaphore to indicate a full buffer.
    */ 
  private Sema full;

  private Object pt;
  private Object gt;

  /**
    * Constructs a BoundedBuffer with a desired size. 
    * @param init An integer, the size of the buffer.
    */
  public BoundedBuffer(int init) {
    buffer = new Object[init];
    size = init;
    front = 0;
    rear = 0;
    empty = new Sema(init);
    full = new Sema(0);
    pt = new Object();
    gt = new Object();
  }


  /**
    * Tests if the BoundedBuffer is empty. 
    * If it is, gives back the value true.
    * @return The boolean value of 
         (whether the BoundedBuffer is empty or not).   
    */
  public boolean empty() {
    return (front == rear);
  }


  /**
   * Add an object to the BoundedBuffer
   * @param v An object.
   */
  public void put(Object v) {
    empty.P();
    synchronized (pt) {
      if (rear >= size) rear -= size;
      buffer[ rear ] = v; 
      rear++;     
    }
    full.V();
  }

  /**
   * Get an object from the BoundedBuffer
   */
  public Object get() {
    Object v;
    full.P();
    synchronized (gt) {
      if (front >= size) front -= size;
      v = buffer[ front ]; 
      front++; 
    }
    empty.V();
    return v;
  }
}
